import gtk
import sys
import pango
from gobject import timeout_add

from gazpacho.loader.loader import ObjectBuilder

from gturing.reader import Reader
from gturing.tape import BLANK

class App(ObjectBuilder):
    def __init__(self, filename):
        ObjectBuilder.__init__(self, filename)
        self.signal_autoconnect(self)
        self.window = self.get_widget('window')
        self.tape_label = self.get_widget('tape_label')
        self.head_label = self.get_widget('head_label')
        self.play_action = self.get_widget('play')
        self.stop_action = self.get_widget('stop')
        self.step_action = self.get_widget('step')
        self.reset_action = self.get_widget('reset')
        self.reader = None
        self.machine = None
        self.tape = ''
        self.speed = 100
        self.running = False

        font_desc = pango.FontDescription('Monospace 10')
        font_size = font_desc.get_size()
        font_desc.set_size(font_size + 4 * pango.SCALE)
        font_desc.set_weight(pango.WEIGHT_BOLD)

        self.tape_label.modify_font(font_desc)
        self.head_label.modify_font(font_desc)

        self.set_sensitive(reset=False, stop=False, 
                           play=False, step=False)

    def on_open__activate(self, action):
        dialog = gtk.FileChooserDialog('Open file...',
                                       self.window,
                                       gtk.FILE_CHOOSER_ACTION_OPEN,
                                       (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                        gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        dialog.set_default_response(gtk.RESPONSE_OK)
        
        filter = gtk.FileFilter()
        filter.set_name('All files')
        filter.add_pattern("*")
        dialog.add_filter(filter)
        
        filter = gtk.FileFilter()
        filter.set_name('Turing programs')
        filter.add_pattern('*.tur')
        dialog.add_filter(filter)

        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            filename = dialog.get_filename()
            self.load_from_file(filename)

        dialog.destroy()

    def on_save__activate(self, action):
        print 'save'

    def on_about__activate(self, action):
        print 'about'

    def on_quit__activate(self, action):
        gtk.main_quit()

    def on_window__delete_event(self, *args):
        gtk.main_quit()

    def do_play(self, *args):
        '''Run the machine each step.  This method is scheduled to 
           run every "speed" microseconds.  It will continue be called 
           if return True.'''
        should_continue = False

        if self.running:
            self.update_tape_label(self.machine.get_tape())
            should_continue = self.machine.run_state()

        if not should_continue:
            self.set_sensitive(reset=True, stop=False, 
                               play=not self.running, 
                               step=not self.running)

        return should_continue

    def on_play__activate(self, action):
        '''Schedule running the machine from the current state each
           "speed" microseconds.'''
        self.running = True
        self.set_sensitive(reset=False, stop=True, 
                           play=False, step=False)

        timeout_add(self.speed, self.do_play, None)

    def on_stop__activate(self, action):
        '''A running machine has been requested to stop by the user'''
        self.running = False
        self.set_sensitive(reset=True, stop=False, 
                           play=True, step=True)

    def on_step__activate(self, action):
        '''Move to the next state (only one)'''
        should_continue = self.machine.run_state()
        self.update_tape_label(self.machine.get_tape())

        if not should_continue:
            # No more states to be processed
            self.set_sensitive(reset=True, stop=False, 
                               play=False, step=False)

    def on_reset__activate(self, action):
        '''Go to the initial state of the machine and go back
           to the initial tape'''
        if self.machine:
            self.machine.reset()
            self.machine.set_tape(self.tape)
            self.update_tape_label(self.tape)

        self.set_sensitive(reset=False, stop=False, 
                           play=True, step=True)

    def on_tape__activate(self, action):
        print 'set tape'

    def on_speed__activate(self, action):
        print 'set speed'

    def on_comments__activate(self, action):
        print 'comments'

    def update_tape_label(self, tape):
        self.tape_label.set_text(tape)
        if self.machine:
            position = self.machine.get_tape_position()
            head = ' '.ljust(len(tape))
            head = head[:position] + '^' + head[position+1:]
            self.head_label.set_text(head)
        else:
            head = '^'.ljust(len(tape))
            self.head_label.set_text(head)

    def set_tape(self, tape):
        self.tape = tape
        if self.machine:
            self.machine.set_tape(tape)
        self.update_tape_label(self.tape)

    def load_from_file(self, filename):
        title = '%s - %s' % (filename, 'Turing Simulator')
        self.window.set_title(title)

        self.reader = Reader(filename)
        self.machine = self.reader.read_states()

        # FIXME: Could we avoid resetting the tape and run the next
        # machine with the current tape?
        self.machine.set_tape(self.tape)
        self.update_tape_label(self.tape)

        self.set_sensitive(reset=False, stop=False, 
                           play=True, step=True)
        print self.machine.get_comments()
        print self.machine

    def set_sensitive(self, reset, stop, play, step):
        self.reset_action.set_sensitive(reset)
        self.stop_action.set_sensitive(stop)
        self.play_action.set_sensitive(play)
        self.step_action.set_sensitive(step)

def main_gui(options, args=[]):
    wt = App('gturing/gturing.glade')

    if options.program:
        wt.load_from_file(options.program)

    if len(args) > 1:
        wt.set_tape(BLANK.join(args[1:]))
    else:
        wt.set_tape('Turing Simulator')

    window = wt.get_widget('window')
    window.set_size_request(400, 300)
    window.show()
    gtk.main()

if __name__ == '__main__':
    sys.exit(main(None, sys.argv))

# vi:set sw=4 ts=4 expandtab:
